package fishrungames.bashgid;


import it.sephiroth.android.library.widget.HListView;

import java.util.ArrayList;

import fishrungames.bashgid.core.AlbumManager.AlbumFullData;
import fishrungames.bashgid.core.ArticleManager.ArticleFullData;
import fishrungames.bashgid.core.PhotoManager.PhotoRecordData;
import fishrungames.networkutils.ImageManager;
import android.annotation.SuppressLint;
import android.content.Context;
import android.content.Intent;
import android.net.Uri;
import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.view.LayoutInflater;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.ViewGroup;
import android.widget.BaseAdapter;

import android.widget.Button;
import android.widget.ImageView;
import android.widget.ListView;
import android.widget.TextView;


public class ArticleFragment extends Fragment
{
	
	public static class ArticleLayoutElement
	{
		public int type; //0 title; 1 text; 2 photo; 3 geotag; 4 photo_album; 5 button
		public String value;
		
		
		public double geoLat;
		public double geoLon;
		
		public int albumPos;
		
		public ArticleLayoutElement(int type, String value)
		{
			this.type = type;
			this.value = value;
			this.geoLat = 0;
			this.geoLon = 0;
			this.albumPos = -1;
		}
		
		
		public ArticleLayoutElement(double geoLat, double geoLon)
		{
			this.type = 3;
			this.value = "";
			this.geoLat = geoLat;
			this.geoLon = geoLon;
			this.albumPos = -1;
		}
		
		public ArticleLayoutElement(int albumPos)
		{
			this.type = 4;
			this.value = "";
			this.geoLat = 0;
			this.geoLon = 0;
			this.albumPos = albumPos;
		}
	}
	
	
	
	ListView listView;
	ListAdapter listAdapter;
	String articleName;
	ArticleFullData article;
	ArrayList<ArticleLayoutElement> articleLayout = new ArrayList<ArticleLayoutElement>();
	
	public ArticleFragment()
	{
		articleName = "";
		article = null;
	}
	
	public ArticleFragment(String articleName)
	{
		this.articleName = articleName;
		RestoreArticle();
	}
	
	@Override
    public void onSaveInstanceState(final Bundle outState) {
        super.onSaveInstanceState(outState);
        outState.putString("articleName", articleName);
    }
	
	public void RestoreArticle()
	{
		article = MainActivity.getInstance().articleDataSource.GetArticleFullData(articleName);
		
		articleLayout.clear();
		
		articleLayout.add(new ArticleLayoutElement(0, article.title));
		
		PhotoRecordData zeroPhoto = null;
		
		if (article.albumArr.size() > 0)
		{
			if (article.albumArr.get(0).photoRecordArr.size() > 0)
			{
				zeroPhoto = article.albumArr.get(0).photoRecordArr.get(0);
			}
		
		}
		
		if (zeroPhoto != null)
		{
			articleLayout.add(new ArticleLayoutElement(2, zeroPhoto.imageUrl));
		}
		
		
		
		articleLayout.add(new ArticleLayoutElement(1, article.content));
		
		if (article.geoLat != 0 && article.geoLon != 0)
		{
			articleLayout.add(new ArticleLayoutElement(article.geoLat, article.geoLon));
		}
		
		for (int i = 0; i < article.albumArr.size(); i++)
		{
			articleLayout.add(new ArticleLayoutElement(i));
		}
		
		if (article.externalLink.length() > 0)
		{
			articleLayout.add(new ArticleLayoutElement(5, article.externalLink));
		}
		
	}
	

	@Override
	public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
		
		if (savedInstanceState != null) {
			articleName = savedInstanceState.getString("articleName");
			RestoreArticle();
        }
		
		View rootView = inflater.inflate(R.layout.fragment_article_page, container, false);
		
		listView = (ListView) rootView.findViewById(R.id.listView);
		
		listAdapter = new ListAdapter(getActivity());
		
		listView.setAdapter(listAdapter);

		return rootView;
	}
	
	public class ListAdapter extends BaseAdapter {
	    private Context mContext;

	    public ListAdapter(Context c) {
	        mContext = c;
	    }

	    public int getCount() {
	        return articleLayout.size();
	    }

	    public Object getItem(int position) {
	        return null;
	    }

	    public long getItemId(int position) {
	        return 0;
	    }
	    
	    @Override
	    public int getViewTypeCount()
	    {
			return 6;
	    }
	    
	    @Override
	    public int getItemViewType(int position)
	    {
			return articleLayout.get(position).type;
			
	    }
	    

	    // create a new ImageView for each item referenced by the Adapter
	    @SuppressLint("InflateParams")
	    public View getView(final int position, View convertView, ViewGroup parent) {
	        
	    	
	    	View view;
	        if (convertView == null) {  // if it's not recycled, initialize some attributes
	        	if (getItemViewType(position) == 0 || getItemViewType(position) == 1)
	        	{
	        		LayoutInflater inflater = (LayoutInflater) mContext.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
	        		view = inflater.inflate(R.layout.article_text_element, null);
	        	}
	        	else if (getItemViewType(position) == 2)
	        	{
	        		LayoutInflater inflater = (LayoutInflater) mContext.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
		        	view = inflater.inflate(R.layout.article_photo_element, null);
	        	}
	        	else if (getItemViewType(position) == 3)
	        	{
	        		LayoutInflater inflater = (LayoutInflater) mContext.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
		        	view = inflater.inflate(R.layout.article_geotag_element, null);
	        	}
	        	else if (getItemViewType(position) == 4)
	        	{
	        		LayoutInflater inflater = (LayoutInflater) mContext.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
		        	view = inflater.inflate(R.layout.article_album_element, null);
	        	}
	        	else //if (getItemViewType(position) == 5)
	        	{
	        		LayoutInflater inflater = (LayoutInflater) mContext.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
		        	view = inflater.inflate(R.layout.article_button_element, null);
	        	}
	        } else {
	        	view = convertView;
	        }
	        
	        
	        if (getItemViewType(position) == 0)
        	{
	        	TextView textView = (TextView) view.findViewById(R.id.dateTextView);
	        	
	        	textView.setText(articleLayout.get(position).value);
		        
        	}
	        else if (getItemViewType(position) == 1)
        	{
	        	TextView textView = (TextView) view.findViewById(R.id.dateTextView);
	        	
	        	textView.setText(articleLayout.get(position).value);
		        
        	}
	        else if (getItemViewType(position) == 2)
	        {
	        	TextView textView = (TextView) view.findViewById(R.id.dateTextView);
	        	ImageView imageView = (ImageView) view.findViewById(R.id.imageView);
	        	
	        	PhotoRecordData photo = MainActivity.getInstance().photoDataSource.GetPhotoByImageUrl(articleLayout.get(position).value);
	        	
	        	ImageManager.getInstance().ApplyImageToImageView(imageView, photo.imageUrl);
	        		        	
	        	textView.setText(photo.title);
        	}
	        else if (getItemViewType(position) == 3)
        	{
	        	//TextView textView = (TextView) view.findViewById(R.id.textView);
	        	
	        	//textView.setText(articleLayout.get(position).value);
		        
        	}
	        else if (getItemViewType(position) == 4)
        	{
	        	//TextView textView = (TextView) view.findViewById(R.id.textView);
	        	
	        	//textView.setText(articleLayout.get(position).value);
	        	
	        	HListView hListView = (HListView) view.findViewById(R.id.hListView);
		        
		        hListView.setAdapter(new MainPageHListViewAdapters.PhotoAlbumElementAdapter(getActivity(), article.albumArr.get(articleLayout.get(position).albumPos)));
		        
        	}
	        else if (getItemViewType(position) == 5)
        	{
	        	Button button = (Button) view.findViewById(R.id.button);
		        
	        	button.setOnClickListener(new OnClickListener()
	        	{
					@Override
					public void onClick(View v)
					{
						Intent i = new Intent(Intent.ACTION_VIEW);
						i.setData(Uri.parse(articleLayout.get(position).value));
						startActivity(i);
					}
	        	});
        	}
	        
	   

	        return view;
	    }

	}
}
