package fishrungames.bashgid;

import java.text.ParseException;
import java.util.ArrayList;
import java.util.Date;
import java.util.Locale;

import fishrungames.bashgid.core.AlbumManager;

import fishrungames.bashgid.core.AlbumManager.AlbumRecordData;
import fishrungames.bashgid.core.ArticleManager;

import fishrungames.bashgid.core.ArticleManager.ArticleRecordData;
import fishrungames.bashgid.core.BookManager.BookRecord;

import fishrungames.bashgid.core.ArticleManager.ArticleFullData;
import fishrungames.bashgid.core.ArticleManager.ArticleShortData;
import fishrungames.bashgid.core.HtmlDownloadManager.TextFileRecord;
import fishrungames.bashgid.core.NewsDownloadTask;
import fishrungames.bashgid.core.NewsManager.NewsRecord;
import fishrungames.bashgid.core.PhotoManager.PhotoRecordData;
import fishrungames.bashgid.core.VideoManager;
import fishrungames.bashgid.core.VideoManager.VideoRecordData;
import fishrungames.bashgid.core.db.AlbumDataSource;
import fishrungames.bashgid.core.db.ArticleDataSource;
import fishrungames.bashgid.core.db.BashgidSqliteHelper;
import fishrungames.bashgid.core.db.ChannelDataSource;
import fishrungames.bashgid.core.db.JournalDataSource;
import fishrungames.bashgid.core.db.NewsDataSource;
import fishrungames.bashgid.core.db.PhotoDataSource;
import fishrungames.bashgid.core.db.VideoDataSource;
import fishrungames.networkutils.ContextHolder;
import fishrungames.networkutils.ImageDownloadTask;
import fishrungames.networkutils.ImageManager;
import android.support.v7.app.ActionBarActivity;
import android.support.v4.app.Fragment;

import android.content.res.Configuration;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.os.Message;

import android.util.Log;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;

import android.support.v4.widget.DrawerLayout;

public class MainActivity extends ActionBarActivity implements NavigationDrawerFragment.NavigationDrawerCallbacks
{

	public static final String TAG_FROM_MAINFRAGMENT_TO_NEWSRECORDFRAGMENT = "TAG_FROM_MAINFRAGMENT_TO_NEWSRECORDFRAGMENT";
	public static final String TAG_FROM_NEWSLISTFRAGMENT_TO_NEWSRECORDFRAGMENT = "TAG_FROM_NEWSLISTFRAGMENT_TO_NEWSRECORDFRAGMENT";
	public static final String TAG_FROM_SEARCHFRAGMENT_TO_NEWSRECORDFRAGMENT = "TAG_FROM_SEARCHFRAGMENT_TO_NEWSRECORDFRAGMENT";
	public static final String TAG_FROM_BOOKSFRAGMENT_TO_BOOKRECORDFRAGMENT = "TAG_FROM_BOOKSFRAGMENT_TO_BOOKRECORDFRAGMENT";
	public static final String TAG_FROM_VIDEOLISTFRAGMENT_TO_VIDEORECORDFRAGMENT = "TAG_FROM_VIDEOLISTFRAGMENT_TO_VIDEORECORDFRAGMENT";

	public static final String TAG_TO_WELCOMEFRAGMENT = "WelcomeFragment";
	public static final String TAG_TO_SEARCHFRAGMENT = "SearchFragment";
	public static final String TAG_TO_MAINFRAGMENT = "MainFragment";
	public static final String TAG_TO_MAPFRAGMENT = "MapFragment";

	public static final int NEWS_DOWNLOADER_STATE_UPDATE = 10;
	public static final int NEWS_DOWNLOADER_STATE_FINISHED = 11;

	MainPageFragment mainPageFragment = null;
	NewsListFragment newsListFragment = null;

	static MainActivity instance = null;

	private NavigationDrawerFragment mNavigationDrawerFragment = null;

	public BashgidSqliteHelper dbHelper = null;
	
	public NewsDataSource newsDataSource = null;
	public PhotoDataSource photoDataSource = null;
	public AlbumDataSource albumDataSource = null;
	public ArticleDataSource articleDataSource = null;

	public VideoDataSource videoDataSource = null;
	public ChannelDataSource channelDataSource = null;
	
	public JournalDataSource journalDataSource = null;
	
	public Handler mHandler;

	@Override
	protected void onCreate(Bundle savedInstanceState)
	{
		super.onCreate(savedInstanceState);

		setContentView(R.layout.activity_main);

		SetupDrawer();

		ImageManager.getInstance().alwaysUpdateImagesWithoutHash = false;

		getSupportFragmentManager().beginTransaction().add(R.id.fragment_container, new WelcomeFragment(), TAG_TO_WELCOMEFRAGMENT).commit();

		instance = this;
		ContextHolder.setContext(instance);
		
		dbHelper = new BashgidSqliteHelper(this);
		
		newsDataSource = new NewsDataSource(this, dbHelper);
		
		photoDataSource = new PhotoDataSource(this, dbHelper);

		albumDataSource = new AlbumDataSource(this, dbHelper);
		
		articleDataSource = new ArticleDataSource(this, dbHelper);
		
		videoDataSource = new VideoDataSource(this, dbHelper);
		
		channelDataSource = new ChannelDataSource(this, dbHelper);
		
		journalDataSource = new JournalDataSource(this, dbHelper);

		mHandler = new Handler(Looper.getMainLooper())
		{
			@Override
			public void handleMessage(Message inputMessage)
			{

				switch (inputMessage.what)
				{
				case ImageDownloadTask.IMAGE_DOWNLOADER_STATE_UPDATE:
				case ImageDownloadTask.IMAGE_DOWNLOADER_STATE_FINISHED:
					UpdateOnImageDownloaded();
					break;
				case NEWS_DOWNLOADER_STATE_UPDATE:
				case NEWS_DOWNLOADER_STATE_FINISHED:
					UpdateOnNewsDownloaded();
					break;
				default:
					super.handleMessage(inputMessage);
				}
			}

		};
		
		
		/*
		
		photoDataSource.AddOrReplacePhoto(new PhotoRecordData(
				"Vlad",
				"Vlad's photo",
				"http://pp.vk.me/c623226/v623226718/2af84/NeEYYIlnR-I.jpg",
				"",
				0,
				0
				));
		
		photoDataSource.AddOrReplacePhoto(new PhotoRecordData(
				"Lilia",
				"Lilia photo",
				"http://http://pp.vk.me/c622725/v622725384/3290f/hfnoyPHe5y8.jpg",
				"",
				0,
				0
				));
		*/
		albumDataSource.CreateNewAlbum(new AlbumRecordData("TEST", "Vk Photos", "Wow"));
		
		
		Date articleTestDate = new Date();
		Date articleTestDate2 = new Date();
		
		try
		{
			articleTestDate = ArticleManager.iso8601Format.parse("2015-05-30 9:00:00");
			articleTestDate2 = ArticleManager.iso8601Format.parse("2015-05-30 9:30:00");
		} catch (ParseException e)
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		AlbumManager.DownloadAndAddPhotoToAlbum(new PhotoRecordData(
				"Lilia",
				"Lilia photo",
				"http://pp.vk.me/c622725/v622725384/3290f/hfnoyPHe5y8.jpg",
				"",
				0,
				0), "TEST");
		
		AlbumManager.DownloadAndAddPhotoToAlbum(new PhotoRecordData(
				"Vlad",
				"Vlad's photo",
				"http://pp.vk.me/c623226/v623226718/2af84/NeEYYIlnR-I.jpg",
				"",
				0,
				0
				), "TEST");
		
		PhotoRecordData photo = photoDataSource.GetPhotoByImageUrl("http://pp.vk.me/c623226/v623226718/2af84/NeEYYIlnR-I.jpg");
		
		if (photo != null)
		{
			Log.e("aaa", "xxx");
		}
		
		articleDataSource.CreateNewArticle(new ArticleRecordData(
				"TESTART",
				0,
				"Photos",
				"Wow 2 photos",
				50,
				50,
				"http://www.yandex.ru/",
				articleTestDate
				));
		
		articleDataSource.AddAlbumToArticle("TEST", "TESTART");
		
		AlbumManager.AddExistingArticleToJournal("TESTART", "news");
		
		
		AlbumManager.AddNewVideoToChannel(new VideoRecordData(
				VideoManager.VIDEO_TYPE_YOUTUBE,
				"http://www.youtube.com/watch?v=dQw4w9WgXcQ",
				"Rickroll",
				"Mega Rickroll",
				"http://img.youtube.com/vi/dQw4w9WgXcQ/0.jpg",
				""), "main");
		
		AlbumManager.AddNewArticleToJournal(new ArticleRecordData(
				"testArticle2",
				ArticleManager.ARTICLE_TYPE_NORMAL,
				"My article",
				"Wow content lol",
				0,
				0,
				"http://fishrungames.ru",
				articleTestDate2
				), "news");
		
		//articleDataSource.AddAlbumToArticle("TEST", "testArticle2");
		//articleDataSource.AddChannelToArticle("main", "testArticle2");
		
		ArrayList<ArticleShortData> fullData = journalDataSource.GetRecentArticles("news", 2);
		
		for (ArticleShortData d : fullData)
		{
			Log.e("sss", d.title);
		}
		
	}

	public static MainActivity getInstance()
	{
		return instance;

	}
	
	public void DownloadImageArr(String [] imageUrlArr)
	{

		Bundle innerQuery = new Bundle();
						
		innerQuery.putStringArray("imageUrlArr", imageUrlArr);

		ImageDownloadTask task = new ImageDownloadTask();

		task.mHandler = mHandler;

		task.execute(innerQuery);
	}
	
	public void DownloadImageArr(String [] imageUrlArr, String [] imageHashArr)
	{

		Bundle innerQuery = new Bundle();
						
		innerQuery.putStringArray("imageUrlArr", imageUrlArr);
		innerQuery.putStringArray("imageHashArr", imageHashArr);

		ImageDownloadTask task = new ImageDownloadTask();

		task.mHandler = mHandler;

		task.execute(innerQuery);
	}
	
	public void DownloadImage(String imageUrl)
	{
		this.DownloadImageArr(new String []{imageUrl});
	}
	
	public void DownloadImage(String imageUrl, String imageHash)
	{

		this.DownloadImageArr(new String []{imageUrl}, new String []{imageHash});
	}

	public void ClearAllFragmentLinks()
	{
		mainPageFragment = null;
		newsListFragment = null;
	}

	public void StartDownloadImagesTask(ArrayList<String> imageUrlArr, ArrayList<String> imageHashArr)
	{
		ImageDownloadTask task = new ImageDownloadTask();

		task.mHandler = mHandler;

		Bundle query = new Bundle();

		query.putStringArray("imageUrlArr", (String[]) imageUrlArr.toArray());

		task.execute(query);
	}
	
	public void StartDownloadNewsTask()
	{
		NewsDownloadTask task = new NewsDownloadTask();
		
		task.mHandler = mHandler;

		task.execute();
	}

	public void UpdateOnImageDownloaded()
	{
		if (mainPageFragment != null)
		{
			mainPageFragment.RefreshHeader();
		}
		
		if (newsListFragment != null)
		{
			newsListFragment.RefreshNewsList();
		}
	}

	public void UpdateOnNewsDownloaded()
	{
		if (mainPageFragment != null)
		{
			mainPageFragment.RefreshHeader();
		}
		
		if (newsListFragment != null)
		{
			newsListFragment.RefreshNewsList();
		}
	}

	public void OnSelectEnglish(View view)
	{
		Locale.setDefault(Locale.US);
		Configuration config = new Configuration();
		config.locale = Locale.US;
		getBaseContext().getResources().updateConfiguration(config, getBaseContext().getResources().getDisplayMetrics());

		getSupportActionBar().setTitle(R.string.app_name);

		SetupDrawer();

		OpenMainScreen();
	}

	public void OnSelectRussian(View view)
	{
		Locale locale = new Locale("ru");
		Locale.setDefault(locale);
		Configuration config = new Configuration();
		config.locale = locale;
		getBaseContext().getResources().updateConfiguration(config, getBaseContext().getResources().getDisplayMetrics());

		getSupportActionBar().setTitle(R.string.app_name);
		SetupDrawer();

		OpenMainScreen();
	}

	public void OnSelectChinese(View view)
	{
		Locale.setDefault(Locale.CHINA);
		Configuration config = new Configuration();
		config.locale = Locale.CHINA;
		getBaseContext().getResources().updateConfiguration(config, getBaseContext().getResources().getDisplayMetrics());

		getSupportActionBar().setTitle(R.string.app_name);
		SetupDrawer();

		OpenMainScreen();
	}

	public void OnSelectSemiDetails(Integer pos)
	{
		// getSupportFragmentManager().beginTransaction().add(R.id.fragment_container,
		// new SemiDetailsFragment()).addToBackStack(null).commit();
	}

	public void OpenMainScreen()
	{
		ClearAllFragmentLinks();
		
		mainPageFragment = new MainPageFragment();

		getSupportFragmentManager().beginTransaction().replace(R.id.fragment_container, mainPageFragment).commit();

		mNavigationDrawerFragment.EnableDrawer();
		
		StartDownloadNewsTask();
	}

	public void OpenNewsScreen()
	{
		
		ClearAllFragmentLinks();
		
		newsListFragment = new NewsListFragment();

		getSupportFragmentManager().beginTransaction().replace(R.id.fragment_container, newsListFragment).commit();

		mNavigationDrawerFragment.EnableDrawer();
		
	}
	
	public void OpenJournalScreen(String journalName)
	{

		ClearAllFragmentLinks();
	
		JournalFragment journal = new JournalFragment(journalName);
		
		getSupportFragmentManager().beginTransaction().replace(R.id.fragment_container, journal).commit();
		
		mNavigationDrawerFragment.EnableDrawer();
	}

	public void OpenSettingsScreen()
	{
		ClearAllFragmentLinks();

		getSupportFragmentManager().beginTransaction().replace(R.id.fragment_container, new SettingsFragment()).commit();

		mNavigationDrawerFragment.EnableDrawer();
	}

	public void OpenSearchScreen()
	{
		ClearAllFragmentLinks();

		getSupportFragmentManager().beginTransaction().replace(R.id.fragment_container, new SearchFragment(), TAG_TO_SEARCHFRAGMENT).commit();

		mNavigationDrawerFragment.EnableDrawer();
	}

	public void OpenSearchScreen(boolean restorePreviousSearch)
	{
		ClearAllFragmentLinks();

		getSupportFragmentManager().beginTransaction().replace(R.id.fragment_container, new SearchFragment(restorePreviousSearch), TAG_TO_SEARCHFRAGMENT)
				.commit();

		mNavigationDrawerFragment.EnableDrawer();

	}

	public void OpenLinkContentScreen(TextFileRecord contentRecord)
	{
		ClearAllFragmentLinks();

		getSupportFragmentManager().beginTransaction().replace(R.id.fragment_container, new LinkContentFragment(contentRecord), TAG_TO_MAINFRAGMENT).commit();

		mNavigationDrawerFragment.EnableDrawer();
	}

	public void OpenMapScreen()
	{
		ClearAllFragmentLinks();

		// getSupportFragmentManager().beginTransaction().replace(R.id.fragment_container,
		// new MapHolderFragment(), TAG_TO_MAINFRAGMENT).commit();
		getSupportFragmentManager().beginTransaction().replace(R.id.fragment_container, new OsmMapHolderFragment(), TAG_TO_MAINFRAGMENT).commit();

		mNavigationDrawerFragment.EnableDrawer();
	}

	public void OpenStreetViewScreen(double lat, double lon)
	{
		ClearAllFragmentLinks();

		getSupportFragmentManager().beginTransaction().replace(R.id.fragment_container, new StreetViewHolderFragment(lat, lon), TAG_TO_MAPFRAGMENT).commit();

		mNavigationDrawerFragment.EnableDrawer();
	}

	public void OpenPhotoAlbumListScreen()
	{
		ClearAllFragmentLinks();

		getSupportFragmentManager().beginTransaction().replace(R.id.fragment_container, new AlbumListFragment(), TAG_TO_MAINFRAGMENT).commit();

		mNavigationDrawerFragment.EnableDrawer();
	}

	public void OpenPhotoAlbumScreen(String albumName)
	{
		ClearAllFragmentLinks();

		getSupportFragmentManager().beginTransaction().replace(R.id.fragment_container, new PhotoAlbumFragment(albumName), TAG_TO_MAINFRAGMENT).commit();

		mNavigationDrawerFragment.EnableDrawer();
	}

	public void OpenNewsRecordScreen(String articleName, String tag)
	{
		// Xperimental -- addToBackStack provoke error "Class not found". Need
		// to resolve somehow!
		// getSupportFragmentManager().beginTransaction().replace(R.id.fragment_container,
		// new NewsRecordFragment(newsRecord)).addToBackStack(null).commit();

		// I added workaround:

		ClearAllFragmentLinks();

		NewsRecordFragment newsRecordFragment = new NewsRecordFragment(articleName);

		getSupportFragmentManager().beginTransaction().replace(R.id.fragment_container, newsRecordFragment, tag).commit();

		mNavigationDrawerFragment.EnableDrawer();

	}
	public void OpenArticleScreen(String articleName, String tag)
	{
		
		ClearAllFragmentLinks();
		
		ArticleFragment articleFragment = new ArticleFragment(articleName);
		
		getSupportFragmentManager().beginTransaction().replace(R.id.fragment_container, articleFragment, tag).commit();

		mNavigationDrawerFragment.EnableDrawer();
	}

	public void OpenHtmlViewerScreen(String url)
	{
		ClearAllFragmentLinks();

		HtmlViewerFragment htmlViewerFragment = new HtmlViewerFragment();
		htmlViewerFragment.url = url;
		getSupportFragmentManager().beginTransaction().replace(R.id.fragment_container, htmlViewerFragment, TAG_TO_MAINFRAGMENT).commit();

		mNavigationDrawerFragment.EnableDrawer();
	}

	public void OpenBooksScreen()
	{
		ClearAllFragmentLinks();

		getSupportFragmentManager().beginTransaction().replace(R.id.fragment_container, new BooksFragment(), TAG_TO_MAINFRAGMENT).commit();

		mNavigationDrawerFragment.EnableDrawer();

	}

	public void OpenBookRecordScreen(BookRecord bookRecord, String tag)
	{
		// Xperimental -- addToBackStack provoke error "Class not found". Need
		// to resolve somehow!
		// getSupportFragmentManager().beginTransaction().replace(R.id.fragment_container,
		// new NewsRecordFragment(newsRecord)).addToBackStack(null).commit();

		// I added workaround:
		ClearAllFragmentLinks();

		BookRecordFragment bookRecordFragment = new BookRecordFragment(bookRecord);

		getSupportFragmentManager().beginTransaction().replace(R.id.fragment_container, bookRecordFragment, tag).commit();

		mNavigationDrawerFragment.EnableDrawer();
	}

	// TAG_FROM_VIDEOLISTFRAGMENT_TO_VIDEORECORDFRAGMENT

	public void OpenVideoListScreen(String channelName)
	{
		ClearAllFragmentLinks();

		getSupportFragmentManager().beginTransaction().replace(R.id.fragment_container, new VideoListFragment(channelName), TAG_TO_MAINFRAGMENT).commit();

		mNavigationDrawerFragment.EnableDrawer();

	}

	public void OpenVideoRecordScreen(String videoUrl, String tag)
	{
		// Xperimental -- addToBackStack provoke error "Class not found". Need
		// to resolve somehow!
		// getSupportFragmentManager().beginTransaction().replace(R.id.fragment_container,
		// new NewsRecordFragment(newsRecord)).addToBackStack(null).commit();

		// I added workaround:

		ClearAllFragmentLinks();

		VideoRecordFragment videoRecordFragment = new VideoRecordFragment(videoUrl);

		getSupportFragmentManager().beginTransaction().replace(R.id.fragment_container, videoRecordFragment, tag).commit();

		mNavigationDrawerFragment.EnableDrawer();
	}

	public void UpdateMainScreenIfPossible(Bundle result)
	{
		if (mainPageFragment != null)
		{
			mainPageFragment.RefreshHeader();
		}
	}

	@Override
	public void onAttachFragment(Fragment fragment)
	{
		super.onAttachFragment(fragment);

	}

	public void SetupDrawer()
	{

		mNavigationDrawerFragment = (NavigationDrawerFragment) getSupportFragmentManager().findFragmentById(R.id.navigation_drawer);

		if (mNavigationDrawerFragment != null)
		{

			// Set up the drawer.
			mNavigationDrawerFragment.setUp(R.id.navigation_drawer, (DrawerLayout) findViewById(R.id.drawer_layout));
		}
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu)
	{

		MenuInflater inflater = getMenuInflater();
		inflater.inflate(R.menu.global, menu);
		return super.onCreateOptionsMenu(menu);
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item)
	{
		// Handle action bar item clicks here. The action bar will
		// automatically handle clicks on the Home/Up button, so long
		// as you specify a parent activity in AndroidManifest.xml.
		int id = item.getItemId();

		if (id == R.id.action_search)
		{
			if (getSupportFragmentManager().findFragmentByTag(TAG_TO_SEARCHFRAGMENT) == null
					&& getSupportFragmentManager().findFragmentByTag(TAG_TO_WELCOMEFRAGMENT) == null)
			{
				OpenSearchScreen();
			}
			return true;
		}

		if (id == R.id.action_settings)
		{
			return true;
		}
		return super.onOptionsItemSelected(item);
	}

	@Override
	public void onNavigationDrawerItemSelected(int position)
	{
		if (position == 1)
		{
			OpenMainScreen();
		} else if (position == 2)
		{
			OpenNewsScreen();
		} else if (position == 4)
		{
			OpenMapScreen();
		} else if (position == 5)
		{
			OpenVideoListScreen("main");
		} else if (position == 6)
		{
			OpenPhotoAlbumListScreen();
			//OpenPhotoAlbumScreen(albumDataSource.GetAlbumFullData("TEST"));
			
		} else if (position == 7)
		{
			OpenBooksScreen();
		} else if (position == 9)
		{
			OpenSettingsScreen();
		}
	}

	@Override
	public void onBackPressed()
	{

		if (getSupportFragmentManager().findFragmentByTag(TAG_FROM_MAINFRAGMENT_TO_NEWSRECORDFRAGMENT) != null)
		{
			OpenMainScreen();

			return;
		}

		if (getSupportFragmentManager().findFragmentByTag(TAG_FROM_NEWSLISTFRAGMENT_TO_NEWSRECORDFRAGMENT) != null)
		{
			OpenNewsScreen();

			return;
		}

		if (getSupportFragmentManager().findFragmentByTag(TAG_FROM_BOOKSFRAGMENT_TO_BOOKRECORDFRAGMENT) != null)
		{
			OpenBooksScreen();

			return;
		}

		if (getSupportFragmentManager().findFragmentByTag(TAG_FROM_VIDEOLISTFRAGMENT_TO_VIDEORECORDFRAGMENT) != null)
		{
			OpenVideoListScreen("main");

			return;
		}

		if (getSupportFragmentManager().findFragmentByTag(TAG_FROM_SEARCHFRAGMENT_TO_NEWSRECORDFRAGMENT) != null)
		{
			OpenSearchScreen(true);

			return;
		}

		if (getSupportFragmentManager().findFragmentByTag(TAG_TO_SEARCHFRAGMENT) != null)
		{
			OpenMainScreen();

			return;
		}

		if (getSupportFragmentManager().findFragmentByTag(TAG_TO_MAINFRAGMENT) != null)
		{
			OpenMainScreen();

			return;
		}

		if (getSupportFragmentManager().findFragmentByTag(TAG_TO_MAPFRAGMENT) != null)
		{
			OpenMapScreen();

			return;
		}

		super.onBackPressed();

		/*
		 * AddToBackStack is falling. Here is workaround
		 * if(getSupportFragmentManager().getBackStackEntryCount() != 0) {
		 * getSupportFragmentManager().popBackStack(); } else {
		 * super.onBackPressed(); }
		 */
	}

	@Override
	protected void onResume()
	{

		super.onResume();
	}

	@Override
	protected void onPause()
	{
		super.onPause();
	}

}
