package fishrungames.bashgid.core.db;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Locale;
import java.util.concurrent.Semaphore;

import fishrungames.bashgid.core.NewsManager;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.SQLException;
import android.database.sqlite.SQLiteDatabase;
import android.util.Log;

public class NewsDataSource
{

	// Xperimental -- move somewhere
	public static final SimpleDateFormat iso8601Format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss", Locale.US);

	// private SQLiteDatabase database = null;

	private BashgidSqliteHelper dbHelper = null;

	private String[] allColumns = { BashgidSqliteHelper.COLUMN_ID, BashgidSqliteHelper.COLUMN_TITLE, BashgidSqliteHelper.COLUMN_DESCRIPTION,
			BashgidSqliteHelper.COLUMN_IMAGE_URL, BashgidSqliteHelper.COLUMN_PUBDATE };

	public NewsDataSource(Context context, BashgidSqliteHelper dbHelper)
	{

		this.dbHelper = dbHelper;
		
	}

	private void createNewsRecord(SQLiteDatabase database, NewsManager.NewsRecord newsRecord)
	{

		ContentValues values = new ContentValues();

		values.put(BashgidSqliteHelper.COLUMN_TITLE, newsRecord.title);
		values.put(BashgidSqliteHelper.COLUMN_DESCRIPTION, newsRecord.description);
		values.put(BashgidSqliteHelper.COLUMN_IMAGE_URL, newsRecord.imageId);
		values.put(BashgidSqliteHelper.COLUMN_PUBDATE, iso8601Format.format(newsRecord.pubDate));

		database.insert(BashgidSqliteHelper.TABLE_NEWS, null, values);

	}

	public void replaceNews(ArrayList<NewsManager.NewsRecord> newsRecordArr)
	{

		SQLiteDatabase database = dbHelper.getWritableDatabase();

		if (database != null)
		{
			try
			{
			database.delete(BashgidSqliteHelper.TABLE_NEWS, null, null);

			for (NewsManager.NewsRecord newsRecord : newsRecordArr)
			{
				createNewsRecord(database, newsRecord);
			}

			dbHelper.close();
			}
			finally
			{
				dbHelper.close();
			}
		}

	}

	public ArrayList<NewsManager.NewsRecord> getNews()
	{

		SQLiteDatabase database = dbHelper.getWritableDatabase();

		ArrayList<NewsManager.NewsRecord> newsRecordArr = new ArrayList<NewsManager.NewsRecord>();

		if (database != null)
		{
			try
			{
			Cursor cursor = database.query(BashgidSqliteHelper.TABLE_NEWS, allColumns, null, null, null, null, null);

			cursor.moveToFirst();

			while (!cursor.isAfterLast())
			{
				NewsManager.NewsRecord newsRecord = cursorToNewsRecord(cursor);
				newsRecordArr.add(newsRecord);
				cursor.moveToNext();
			}

			cursor.close();

			}
			finally
			{
			dbHelper.close();
			}
		}

		return newsRecordArr;
	}

	public ArrayList<NewsManager.NewsRecord> searchNewsByString(String searchText)
	{

		
		ArrayList<NewsManager.NewsRecord> newsRecordArr = new ArrayList<NewsManager.NewsRecord>();

		SQLiteDatabase database = dbHelper.getWritableDatabase();

		if (database != null)
		{

			try
			{
			Cursor cursor = database.query(true, BashgidSqliteHelper.TABLE_NEWS, allColumns, BashgidSqliteHelper.COLUMN_TITLE + " LIKE '%" + searchText + "%'",
					null, null, null, null, null);

			cursor.moveToFirst();

			while (!cursor.isAfterLast())
			{
				NewsManager.NewsRecord newsRecord = cursorToNewsRecord(cursor);
				newsRecordArr.add(newsRecord);
				cursor.moveToNext();
			}

			cursor.close();
			}
			finally
			{
				dbHelper.close();
			}

			
		}

		return newsRecordArr;
	}

	private NewsManager.NewsRecord cursorToNewsRecord(Cursor cursor)
	{

		NewsManager.NewsRecord newsRecord = null;
		try
		{
			newsRecord = new NewsManager.NewsRecord(cursor.getString(1), cursor.getString(2), cursor.getString(3), iso8601Format.parse(cursor.getString(4)));
		} catch (ParseException e)
		{
			e.printStackTrace();
		}

		return newsRecord;
	}
}
